import { PlainObjectOf, StateClass } from '@ngxs/store/internals';
import { Observable } from 'rxjs';
import { META_KEY, META_OPTIONS_KEY, NgxsConfig, StoreOptions } from '../symbols';
import { ActionHandlerMetaData } from '../actions/symbols';
export interface StateClassInternal<T = any, U = any> extends StateClass<T> {
    [META_KEY]?: MetaDataModel;
    [META_OPTIONS_KEY]?: StoreOptions<U>;
}
export declare type StateKeyGraph = PlainObjectOf<string[]>;
export declare type StatesByName = PlainObjectOf<StateClassInternal>;
export interface StateOperations<T> {
    getState(): T;
    setState(val: T): T;
    dispatch(actionOrActions: any | any[]): Observable<void>;
}
export interface MetaDataModel {
    name: string | null;
    actions: PlainObjectOf<ActionHandlerMetaData[]>;
    defaults: any;
    path: string | null;
    makeRootSelector: SelectorFactory | null;
    children?: StateClassInternal[];
}
export interface RuntimeSelectorContext {
    getStateGetter(key: any): (state: any) => any;
    getSelectorOptions(localOptions?: SharedSelectorOptions): SharedSelectorOptions;
}
export declare type SelectFromRootState = (rootState: any) => any;
export declare type SelectorFactory = (runtimeContext: RuntimeSelectorContext) => SelectFromRootState;
export interface SharedSelectorOptions {
    injectContainerState?: boolean;
    suppressErrors?: boolean;
}
export interface SelectorMetaDataModel {
    makeRootSelector: SelectorFactory | null;
    originalFn: Function | null;
    containerClass: any;
    selectorName: string | null;
    getSelectorOptions: () => SharedSelectorOptions;
}
export interface MappedStore {
    name: string;
    isInitialised: boolean;
    actions: PlainObjectOf<ActionHandlerMetaData[]>;
    defaults: any;
    instance: any;
    path: string;
}
export interface StatesAndDefaults {
    defaults: any;
    states: MappedStore[];
}
/**
 * Ensures metadata is attached to the class and returns it.
 *
 * @ignore
 */
export declare function ensureStoreMetadata(target: StateClassInternal): MetaDataModel;
/**
 * Get the metadata attached to the state class if it exists.
 *
 * @ignore
 */
export declare function getStoreMetadata(target: StateClassInternal): MetaDataModel;
/**
 * Ensures metadata is attached to the selector and returns it.
 *
 * @ignore
 */
export declare function ensureSelectorMetadata(target: Function): SelectorMetaDataModel;
/**
 * Get the metadata attached to the selector if it exists.
 *
 * @ignore
 */
export declare function getSelectorMetadata(target: any): SelectorMetaDataModel;
/**
 * Get a deeply nested value. Example:
 *
 *    getValue({ foo: bar: [] }, 'foo.bar') //=> []
 *
 * @ignore
 */
export declare function propGetter(paths: string[], config: NgxsConfig): (x: any) => any;
/**
 * Given an array of states, it will return a object graph. Example:
 *    const states = [
 *      Cart,
 *      CartSaved,
 *      CartSavedItems
 *    ]
 *
 * would return:
 *
 *  const graph = {
 *    cart: ['saved'],
 *    saved: ['items'],
 *    items: []
 *  };
 *
 * @ignore
 */
export declare function buildGraph(stateClasses: StateClassInternal[]): StateKeyGraph;
/**
 * Given a states array, returns object graph
 * returning the name and state metadata. Example:
 *
 *  const graph = {
 *    cart: { metadata }
 *  };
 *
 * @ignore
 */
export declare function nameToState(states: StateClassInternal[]): PlainObjectOf<StateClassInternal>;
/**
 * Given a object relationship graph will return the full path
 * for the child items. Example:
 *
 *  const graph = {
 *    cart: ['saved'],
 *    saved: ['items'],
 *    items: []
 *  };
 *
 * would return:
 *
 *  const r = {
 *    cart: 'cart',
 *    saved: 'cart.saved',
 *    items: 'cart.saved.items'
 *  };
 *
 * @ignore
 */
export declare function findFullParentPath(obj: StateKeyGraph, newObj?: PlainObjectOf<string>): PlainObjectOf<string>;
/**
 * Given a object graph, it will return the items topologically sorted Example:
 *
 *  const graph = {
 *    cart: ['saved'],
 *    saved: ['items'],
 *    items: []
 *  };
 *
 * would return:
 *
 *  const results = [
 *    'items',
 *    'saved',
 *    'cart'
 *  ];
 *
 * @ignore
 */
export declare function topologicalSort(graph: StateKeyGraph): string[];
/**
 * Returns if the parameter is a object or not.
 *
 * @ignore
 */
export declare function isObject(obj: any): boolean;
